/* ###*B*###
 * ERIKA Enterprise - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2009  Evidence Srl
 *
 * This file is part of ERIKA Enterprise.
 *
 * ERIKA Enterprise is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation, 
 * (with a special exception described below).
 *
 * Linking this code statically or dynamically with other modules is
 * making a combined work based on this code.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * ERIKA Enterprise is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 * ###*E*### */

/*
 * Erika types, based on standard types defined in <limits.h>
 * Author: 2009 Bernardo Dal Seno
 */

#ifndef __INCLUDE_CPU_COMMON_EE_TYPES__
#define __INCLUDE_CPU_COMMON_EE_TYPES__

/* For Gcc we could use __XXX_MAX__ built-in macros, instead */
#include <limits.h>

/* Primitive data types */
typedef unsigned char EE_BIT;

#if SCHAR_MAX == 127
typedef unsigned char EE_UINT8;
typedef signed char EE_INT8;
#else
#error Unable to define a 8-bit types
#endif

#if INT_MAX == 32767 /* 16-bit architecture */
typedef int EE_INT16;
typedef unsigned int EE_UINT16;
typedef long int EE_INT32;
typedef unsigned long int EE_UINT32;
#elif (INT_MAX == 2147483647) && (SHRT_MAX == 32767) /* 32-bit architecture */
typedef short int EE_INT16;
typedef unsigned short int EE_UINT16;
typedef int EE_INT32;
typedef unsigned int EE_UINT32;
#else /* Unknown architecture */
#error Unknow/unsupported architecture
#endif

/* HAL may need to declare pointers using keywords such as `far'... */
#ifndef __EE_HAL_CUSTOM_POINTERS__

/* Data addresses (that have the same size of a pointer) */
typedef void *EE_ADDR;

/* Code addresses (same size of function pointers) */
typedef void (*EE_FADDR)(void);

/* Callbacks with no parameters nor return value */
typedef void (*EE_VOID_CALLBACK)(void);

/* Callback function called by driver handlers */
typedef EE_VOID_CALLBACK EE_ISR_callback;

#endif /* __EE_HAL_CUSTOM_POINTERS__ */

#endif /* __INCLUDE_CPU_COMMON_EE_TYPES__ */
