#******************************************************************************
#                                                                             *
# License Agreement                                                           *
#                                                                             *
# Copyright (c) 2004 Altera Corporation, San Jose, California, USA.           *
# All rights reserved.                                                        *
#                                                                             *
# Permission is hereby granted, free of charge, to any person obtaining a     *
# copy of this software and associated documentation files (the "Software"),  *
# to deal in the Software without restriction, including without limitation   *
# the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
# and/or sell copies of the Software, and to permit persons to whom the       *
# Software is furnished to do so, subject to the following conditions:        *
#                                                                             *
# The above copyright notice and this permission notice shall be included in  *
# all copies or substantial portions of the Software.                         *
#                                                                             *
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
# DEALINGS IN THE SOFTWARE.                                                   *
#                                                                             *
# This agreement shall be governed in all respects by the laws of the State   *
# of California and by the laws of the United States of America.              *
#                                                                             *
# Altera does not recommend, suggest or require that this reference design    *
# file be used in conjunction or combination with any other product.          *
#******************************************************************************

#******************************************************************************
#                                                                             *
# THIS IS A LIBRARY READ-ONLY SOURCE FILE. DO NOT EDIT.                       *
#                                                                             *
#******************************************************************************

# This file defines the rules used for generating files which are dependent on 
# the STF and PTF files, i.e. generated_app.mk, generated_all.mk, generated.sh,
# alt_sys_init.c and system.h.

# The GTF files are places in a fixed location relative to the nios2 kit 
#directory.

GTF = $(SOPC_KIT_NIOS2_M)/bin/gtf

# Directory for the timestamp files

TIMESTAMP_DIR = $(GTF_GENERATED)/../$(OBJ_DIR)

# To avoid excessive rebuilds, these generated files are only dependent on this
# makefile, not on the complete $(MAKEFILE_LIST).

GTF_DEPS = $(PTF) $(STF) \
           $(SOPC_KIT_NIOS2)/components/evidence_ee/build/gtf_rules.mk

# get our path converter as cygpath (for windows) or echo (no conversion on 
# unix)

CYGPATHEXE = $(shell if [ "X`which cygpath.exe 2> /dev/null`" != "X" ]; \
             then echo "cygpath -d"; else echo "echo"; fi)

# install.ptf is in this directory

SOPC_HOME_DIR = $(dir $(PTF)).sopc_builder

# PTF file in mixed format

PTF_M = $(shell cygpath -m "$(PTF)")

# Rules for generating the various files generated by gtf-generate.
#
# For each generated file there is actually a pair of rules. This is because 
# gtf-generate will only touch the output file if the contents have changed
# since the last visit. By making the output file dependent on a "timestamp
# file", which is in turn generated as a side effect of running gtf-generate,
# we save having to run gtf-generate unecessarily when we know the 
# output is up to date. 

$(GTF_GENERATED)/generated.x      \
$(GTF_GENERATED)/generated_app.mk \
$(GTF_GENERATED)/generated_all.mk \
$(GTF_GENERATED)/generated.sh     \
$(GTF_GENERATED)/generated.gdb    \
$(GTF_GENERATED)/system.h         \
$(GTF_GENERATED)/alt_sys_init.c: $(GTF_GENERATED)/%: $(TIMESTAMP_DIR)/%-t
	@true

$(TIMESTAMP_DIR)/generated.x-t      \
$(TIMESTAMP_DIR)/generated_app.mk-t \
$(TIMESTAMP_DIR)/generated.sh-t     \
$(TIMESTAMP_DIR)/generated.gdb-t    \
$(TIMESTAMP_DIR)/system.h-t: $(TIMESTAMP_DIR)/%-t: \
                                   $(GTF)/%.gtf $(GTF_DEPS) 
	@echo Creating $(patsubst %-t,%, $(notdir $@))...
	@mkdir -p $(GTF_GENERATED)
	@mkdir -p $(dir $@)
	SOPC_KIT_NIOS2='$(SOPC_KIT_NIOS2)' gtf-generate \
          '--gtf=$<' '--stf=$(STF)'                 \
          '--output-directory=$(GTF_GENERATED)'
	@echo This is an auto-generated timestamp file. > $@

# The two files: "generated_all.mk" and "alt_sys_init.c" differ from the
# above in that they are dependent on the file install.ptf. This file lists
# the location of all available components. Therefore this file is brought
# up to date before "generated_all.mk" or "alt_sys_init.c" are generated.

$(TIMESTAMP_DIR)/generated_all.mk-t \
$(TIMESTAMP_DIR)/alt_sys_init.c-t: $(TIMESTAMP_DIR)/%-t: \
                                   $(GTF)/%.gtf $(GTF_DEPS) \
                                   $(SOPC_HOME_DIR)/install.ptf
	@echo Creating $(patsubst %-t,%, $(notdir $@))...
	@mkdir -p $(GTF_GENERATED)
	@mkdir -p $(dir $@)
	SOPC_KIT_NIOS2='$(SOPC_KIT_NIOS2)' gtf-generate \
          '--gtf=$<' '--stf=$(STF)'                 \
          '--output-directory=$(GTF_GENERATED)'
	@echo This is an auto-generated timestamp file. > $@

$(SOPC_HOME_DIR)/install.ptf: $(PTF) $(TIMESTAMP_DIR)/install.ptf-t
	"$(QUARTUS_ROOTDIR)/sopc_builder/bin/sopc_builder" \
                 --update_classes_and_exit --no_splash -s \
                 --projectpath=$(dir $(PTF_M)); \
                 [ $$? = 2 ]
	@touch $(SOPC_HOME_DIR)/install.ptf

$(TIMESTAMP_DIR)/install.ptf-t:
	@mkdir -p $(dir $@)
	@echo This is an auto-generated timestamp file. > $@
